@@ -83,36 +83,6 @@ def test_systeminit():
     ]
 
 
-def test_chi2lnlike():
-    """
-    Test the ability of ``orbitize.lnlike.chi2_lnlike()``
-    to work properly on arrays.
-    """
-    # test with a single model
-    model = np.zeros((3, 2))
-    jitter = np.zeros((3, 2))
-    data = np.ones((3, 2))
-    errors = np.ones((3, 2))
-
-    seppa_indices = [np.array([1])]
-
-    chi2 = lnlike.chi2_lnlike(data, errors, model, jitter, seppa_indices)
-    assert chi2.shape == (3, 2)
-    assert (chi2 == -0.5 * np.ones((3, 2)) - np.log(np.sqrt(2*np.pi*np.ones((3, 2))))).all()
-
-    # test with multiple models
-    model = np.zeros((3, 2, 5))
-    jitter = np.zeros((3, 2, 5))
-    data = np.ones((3, 2))
-    errors = np.ones((3, 2))
-
-    seppa_indices = [np.array([1])]
-
-    chi2 = lnlike.chi2_lnlike(data, errors, model, jitter, seppa_indices)
-    assert chi2.shape == (3, 2, 5)
-    assert (chi2 == -0.5 * np.ones((3, 2, 5)) - np.log(np.sqrt(2*np.pi*np.ones((3, 2, 5))))).all()
-
-
 def test_custom_likelihood():
     """
     Tests the inclusion of a custom likelihood function in the code
